@@ -7,3 +7,12 @@ __maintainer__ = "Thomas LECOCQ"
 __email__ = "Thomas.Lecocq at seismology.be"
 __status__ = "Production"
 
+
+class MSNoiseError(Exception):
+    pass
+
+class DBConfigNotFoundError(MSNoiseError):
+    pass
+
+class FatalError(MSNoiseError):
+    pass
