@@ -1,7 +1,7 @@
 """MSNoise Admin is a web interface that helps the user define the
-configuration for all the processing steps, it also allows configuring the
-stations and filters to be used in the processes. It gives a view on the
-database tables.
+configuration for all the processing steps. It allows configuring the
+stations and filters to be used in the different steps of the workflow and
+provides a view on the database tables.
 
 To start the admin:
 
@@ -10,7 +10,7 @@ To start the admin:
     $ msnoise admin
 
 Which, by default, starts a web server listening on all interfaces on port
-5000. This can be overriden by passing parameters to the command, e.g. for port
+5000. This can be overridden by passing parameters to the command, e.g. for port
 5099:
 
 .. code-block:: sh
@@ -18,8 +18,8 @@ Which, by default, starts a web server listening on all interfaces on port
     $ msnoise admin -p 5099
 
 
-Next step is to open a web browser and open the ip address of the machine,
-by default on the current machine, it'll be http://localhost:5000/ or
+The next step consists of opening a web browser and open the ip address of the
+machine, by default on the current machine, it'll be http://localhost:5000/ or
 http://127.0.0.1:5000/.
 
 .. image:: ../.static/msnoise_admin_home.png
@@ -34,9 +34,10 @@ Home
 The index page shows
 
 * The project location and its database
-* Stats of the Data Availability, the CC jobs and the DTT jobs
+* Stats of the Data Availability, the CC, STACK, MWCS and DTT jobs.
+* Quick action buttons for resetting or deleting jobs.
 
-The name and the logo of the page can be overriden by setting an environment
+The name and the logo of the page can be overridden by setting an environment
 variable with a name and the HTML tag of the logo image:
 
 .. code:: sh
@@ -86,7 +87,7 @@ Example view:
 .. image:: ../.static/msnoise_admin_config.png
     :align: center
 
-The table below repeats this
+The table below lists the different fields:
 
 .. include:: ../defaults.rst
 
@@ -147,7 +148,8 @@ from flask_wtf import Form
 
 from .api import *
 from .default import default
-from .msnoise_table_def import *
+
+from .msnoise_table_def import Filter, Job, Station, Config, DataAvailability
 
 
 class GenericView(BaseView):
@@ -224,7 +226,8 @@ class StationView(ModelView):
     view_title = "Station Configuration"
     column_filters = ('net', 'used')
     column_type_formatters = MY_DEFAULT_FORMATTERS
-    
+    can_set_page_size = True
+
     def __init__(self, session, **kwargs):
         super(StationView, self).__init__(Station, session, **kwargs)
     
@@ -249,6 +252,7 @@ class DataAvailabilityView(ModelView):
     can_delete = True
     can_edit = True
     page_size = 100
+    can_set_page_size = True
     column_filters = ('net', 'sta', 'comp', 'data_duration', 'gaps_duration',
                       'samplerate', 'flag')
 
@@ -281,6 +285,7 @@ class JobView(ModelView):
     column_filters = ('pair', 'jobtype', 'flag')
     page_size = 100
     edit_modal = True
+    can_set_page_size = True
 
     def __init__(self, session, **kwargs):
         super(JobView, self).__init__(Job, session, **kwargs)
@@ -343,6 +348,8 @@ class ConfigView(ModelView):
     can_create = False
     can_delete = False
     page_size = 100
+    can_set_page_size = True
+    column_sortable_list = ["name",]
 
     # Override displayed fields
     column_list = ('name', 'value')
@@ -367,6 +374,12 @@ class ConfigView(ModelView):
         self._template_args['helpstringdefault'] = default[id][1]
         return super(ConfigView, self).edit_view()
 
+    def on_model_change(self, form, model, is_created):
+        if form.data['value']:
+            model.value = form.data['value'].strip()
+        else:
+            model.value = ''
+
 
 def getitem(obj, item, default):
     if item not in obj:
@@ -418,18 +431,19 @@ class ResultPlotter(BaseView):
         format = getitem(args, 'format', 'stack')
 
         db = connect()
+        params = get_params(db)
         station1, station2 = pairs[pair]['text'].replace('.', '_').split(' - ')
-        start, end, dates = build_ref_datelist(db)
+        start, end, dates = build_movstack_datelist(db)
+        print(station1, station2, filter, component, dates, format)
         i, result = get_results(db,station1, station2, filter, component, dates,
-                                format=format)
+                                format=format, params=params)
 
         if format == 'stack':
             if i != 0:
-                maxlag = float(get_config(db, 'maxlag'))
-                x = np.linspace(-maxlag, maxlag, get_maxlag_samples(db))
+                x = get_t_axis(db)
                 y = result
         db.close()
-
+        # print(result)
         # fig = figure(title=pairs[pair]['text'], plot_width=1000)
         # fig.line(x, y, line_width=2)
         #
@@ -441,12 +455,27 @@ class ResultPlotter(BaseView):
         # )
         #
         # script, div = components(fig, INLINE)
-        # return self.render(
-        #     'admin/results.html',
-        #     plot_script=script, plot_div=div, plot_resources=plot_resources,
-        #     filter_list=filters,
-        #     pair_list=pairs
-        # )
+        import matplotlib.pyplot as plt
+        plt.figure(figsize=(12,5))
+        plt.plot(x, y)
+
+        from io import BytesIO
+        figfile = BytesIO()
+        plt.savefig(figfile, format='png')
+        figfile.seek(0)  # rewind to beginning of file
+        import base64
+        # figdata_png = base64.b64encode(figfile.read())
+        figdata_png = base64.b64encode(figfile.getvalue())
+        result= figdata_png
+
+        plot_div = ""
+
+        return self.render(
+            'admin/results.html',
+            plot_script="", plot_div=plot_div, plot_resources="",
+            filter_list=filters,
+            pair_list=pairs, result=result.decode('utf8')
+        )
 
 
 class InterferogramPlotter(BaseView):
@@ -609,7 +638,7 @@ def new_jobsTRIG():
 def joblists():
     jobtype = flask.request.args['type']
     db = connect()
-    data = get_job_types(db,jobtype)
+    data = get_job_types(db, jobtype)
     db.close()
     o = {'T': 0, 'I': 0, 'D': 0}
     for count, flag in data:
@@ -618,6 +647,18 @@ def joblists():
     return flask.Response(o, mimetype='application/json')
 
 
+@app.route('/admin/resetjobs.json')
+def resetjobs():
+    jobtype = flask.request.args['type']
+    alljobs = flask.request.args['all']
+    from msnoise.api import reset_jobs
+    reset_jobs(db, jobtype, alljobs=eval(alljobs))
+    o = {}
+    o["Done"] = "Jobs reset: Done."
+    o = json.dumps(o)
+    return flask.Response(o, mimetype='application/json')
+
+
 @app.route('/admin/data_availability.png')
 def DA_PNG():
     from .plots.data_availability import main
@@ -675,14 +716,19 @@ def main(port=5000):
     else:
         admin.name = "MSNoise"
     admin.project_folder = os.getcwd()
-    tech, hostname, database, username, password = read_database_inifile()
-    if tech == 1:
-        database = "SQLite: %s" % hostname
+    dbini = read_db_inifile()
+    if dbini.prefix != "":
+        prefix = "/%s_*" % dbini.prefix
     else:
-        database = "MySQL: %s@%s:%s" % (username, hostname, database)
+        prefix = ""
+    if dbini.tech == 1:
+        database = "SQLite: %s%s" % (dbini.hostname, prefix)
+    else:
+        database = "MySQL: %s@%s:%s%s" % (dbini.username, dbini.hostname,
+                                          dbini.database, prefix)
     admin.project_database = database
 
-    jobtypes = ["CC", "DTT"]
+    jobtypes = ["CC", "STACK", "MWCS", "DTT"]
     template_folders = []
     if plugins:
         for ep in pkg_resources.iter_entry_points(group='msnoise.plugins.jobtypes'):
@@ -714,11 +760,11 @@ def main(port=5000):
     admin.add_view(JobView(db,endpoint='jobs',category='Database'))
 
 
-    # admin.add_view(DataAvailabilityPlot(endpoint='data_availability_plot',
-                                        # category='Results'))
-    # admin.add_view(ResultPlotter(endpoint='results',category='Results'))
-    # admin.add_view(InterferogramPlotter(endpoint='interferogram',
-                                        # category='Results'))
+    admin.add_view(DataAvailabilityPlot(endpoint='data_availability_plot',
+                                        category='Results'))
+    admin.add_view(ResultPlotter(endpoint='results',category='Results'))
+    admin.add_view(InterferogramPlotter(endpoint='interferogram',
+                                        category='Results'))
 
     if plugins:
         plugins = plugins.split(',')
