@@ -22,42 +22,38 @@ Configuration Parameters
 * |response_prefilt|
 * |preprocess_lowpass|
 * |preprocess_highpass|
+* |preprocess_max_gap|  | *new in 1.6*
 * |keep_all|
 * |keep_days|
 * |stack_method|
 * |pws_timegate|
 * |pws_power|
 * |whitening|  | *new in 1.5*
+* |hpc| | *new in 1.6*
 
 Waveform Pre-processing
 ~~~~~~~~~~~~~~~~~~~~~~~
 Pairs are first split and a station list is created. The database is then
 queried to get file paths. For each station, all files potentially containing
-data for the day are opened. The traces are then merged and splitted, to obtain
+data for the day are opened. The traces are then merged and split, to obtain
 the most continuous chunks possible. The different chunks are then demeaned,
 tapered and merged again to a 1-day long trace. If a chunk is not aligned
 on the sampling grid (that is, start at a integer times the sample spacing in s)
 , the chunk is phase-shifted in the frequency domain. This requires tapering and
-fft/ifft. If the gap between two chunks is small, compared to a currently
-hard-coded value (10 samples), the gap is filled with interpolated values.
+fft/ifft. If the gap between two chunks is small, compared to a configurable
+value (``preprocess_max_gap``), the gap is filled with interpolated values.
 Larger gaps will not be filled with interpolated values.
 
 .. warning::
     As from MSNoise 1.5, traces are no longer padded by or merged with 0s.
 
-Each 1-day long trace is then low-passed (at ``preprocess_lowpass`` Hz),
-high-passed (at ``preprocess_highpass`` Hz), then if needed,
+Each 1-day long trace is then high-passed (at ``preprocess_highpass`` Hz),
+then if needed low-passed (at ``preprocess_lowpass`` Hz) and
 decimated/downsampled. Decimation/Downsampling are configurable
 (``resampling_method``) and users are advised testing Decimate. One advantage of
 Downsampling over Decimation is that it is able to downsample the data by any
 factor, not only integer factors. Downsampling can be achieved with the new
-ObsPy Lanczos resampler, giving results similar to those by scikits.samplerate.
-
-.. note:: Python 3 users will most probably struggle installing
-    scikits.samplerate, and therefore will have to use either Decimate or
-    Lanczos instead of Resample. This is not a problem because the Lanczos
-    resampling gives results similar to those by scikits.samplerate.
-
+ObsPy Lanczos resampler.
 
 If configured, each 1-day long trace is corrected for its instrument response.
 Currently, only dataless seed and inventory XML are supported.
@@ -118,7 +114,8 @@ value is 2.
     of weak, coherent signals through phase-weighted stacks". Geophysical
     Journal International 130, 2 (1997): 497-505.
 
-Once done, each job is marked "D"one in the database.
+Once done, each job is marked "D"one in the database and, unless ``hpc`` is
+``Y``, STACK jobs are inserted/updated in the database.
 
 To run this script:
 
@@ -151,15 +148,16 @@ could occur with SQLite.
     The preprocessing routine is separated from the compute_cc and can be called
     by external plugins.
 
+.. versionadded:: 1.6
+    The ``preprocess_max_gap`` parameter.
+    The ``hpc`` parameter that can prevent the automatic creation of STACK jobs.
+    
+
+
 """
 import sys
 import time
 
-try:
-    from scikits.samplerate import resample
-except:
-    pass
-
 from .api import *
 from .move2obspy import myCorr
 from .move2obspy import whiten
@@ -167,56 +165,42 @@ from .move2obspy import whiten
 from .preprocessing import preprocess
 
 
-class Params():
-    pass
+import logbook
 
 
-def main():
-    logging.basicConfig(level=logging.INFO,
-                        format='%(asctime)s [%(levelname)s] %(message)s',
-                        datefmt='%Y-%m-%d %H:%M:%S')
+def main(loglevel="INFO"):
+    logger = logbook.Logger("msnoise")
+    # Reconfigure logger to show the pid number in log records
+    logger = get_logger('msnoise.compute_cc_child', loglevel,
+                        with_pid=True)
 
-    logging.info('*** Starting: Compute CC ***')
+    logger.info('*** Starting: Compute CC ***')
 
     # Connection to the DB
     db = connect()
 
     if len(get_filters(db, all=False)) == 0:
-        logging.info("NO FILTERS DEFINED, exiting")
+        logger.info("NO FILTERS DEFINED, exiting")
         sys.exit()
 
     # Get Configuration
-    params = Params()
-    params.goal_sampling_rate = float(get_config(db, "cc_sampling_rate"))
-    params.goal_duration = float(get_config(db, "analysis_duration"))
-    params.overlap = float(get_config(db, "overlap"))
-    params.maxlag = float(get_config(db, "maxlag"))
-    params.corr_duration = float(get_config(db, "corr_duration"))
-    params.min30 = float(get_config(db, "corr_duration")) *\
-                   params.goal_sampling_rate
-    params.windsorizing = float(get_config(db, "windsorizing"))
-    params.whitening = get_config(db, 'whitening')
-    params.resampling_method = get_config(db, "resampling_method")
-    params.preprocess_lowpass = float(get_config(db, "preprocess_lowpass"))
-    params.preprocess_highpass = float(get_config(db, "preprocess_highpass"))
-    params.keep_all = get_config(db, 'keep_all', isbool=True)
-    params.keep_days = get_config(db, 'keep_days', isbool=True)
-    params.components_to_compute = get_components_to_compute(db)
+    params = get_params(db)
+    filters = get_filters(db, all=False)
 
-    params.stack_method = get_config(db, 'stack_method')
-    params.pws_timegate = float(get_config(db, 'pws_timegate'))
-    params.pws_power = float(get_config(db, 'pws_power'))
+    logger.info("Will compute %s" % " ".join(params.components_to_compute))
+    
+    if "R" not in ''.join(params.components_to_compute) and "T" not in ''.join(params.components_to_compute):
+        logger.info("You seem to have configured no R nor T components, thus no rotation are needed. You should therefore use the 'msnoise compute_cc' instead, which is much faster")
 
-    logging.info("Will compute %s" % " ".join(params.components_to_compute))
-
-    if get_config(db, 'remove_response', isbool=True):
-        logging.debug('Pre-loading all instrument response')
+    if params.remove_response:
+        logger.debug('Pre-loading all instrument response')
         responses = preload_instrument_responses(db)
     else:
         responses = None
 
     while is_next_job(db, jobtype='CC'):
         jobs = get_next_job(db, jobtype='CC')
+
         stations = []
         pairs = []
         refs = []
@@ -228,34 +212,37 @@ def main():
             stations.append(netsta1)
             stations.append(netsta2)
             goal_day = job.day
-        del jobs
 
         stations = np.unique(stations)
 
-        logging.info("New CC Job: %s (%i pairs with %i stations)" %
+        logger.info("New CC Job: %s (%i pairs with %i stations)" %
                      (goal_day, len(pairs), len(stations)))
         jt = time.time()
 
         comps = []
-        for comp in params.components_to_compute:
-            if comp[0] in ["R", "T"] or comp[1] in ["R", "T"]:
-                comps.append("E")
-                comps.append("N")
-            else:
+        for comp in params.all_components:
+            if comp[0] in ["Z", "E", "N", "1", "2"]:
                 comps.append(comp[0])
+            if comp[1] in ["Z", "E", "N", "1", "2"]:
                 comps.append(comp[1])
+            for c in comp:
+                if c in ["R", "T"]:
+                    comps.append("E")
+                    comps.append("N")
+
         comps = np.unique(comps)
-        basetime, stream = preprocess(db, stations, comps, goal_day, params,
-                                      responses)
+        stream = preprocess(db, stations, comps, goal_day, params, responses)
 
         # print '##### STREAMS ARE ALL PREPARED AT goal Hz #####'
         dt = 1. / params.goal_sampling_rate
-
+        start_processing = time.time()
         # ITERATING OVER PAIRS #####
-        for pair in pairs:
+        logger.info("Will do %i pairs" % len(pairs))
+        for job in jobs:
+            pair = job.pair
             orig_pair = pair
 
-            logging.info('Processing pair: %s' % pair.replace(':', ' vs '))
+            logger.info('Processing pair: %s' % pair.replace(':', ' vs '))
             tt = time.time()
             station1, station2 = pair.split(':')
             pair = (np.where(stations == station1)
@@ -265,21 +252,21 @@ def main():
             s2 = get_station(db, station2.split('.')[0], station2.split('.')[1])
 
             if s1.X and params.components_to_compute != ["ZZ", ]:
-                X0, Y0, c0 = (s1.X, s1.Y, s1.coordinates)
-                X1, Y1, c1 = (s2.X, s2.Y, s1.coordinates)
+                x0, y0, c0 = (s1.X, s1.Y, s1.coordinates)
+                x1, y1, c1 = (s2.X, s2.Y, s1.coordinates)
 
                 if c0 == c1:
                     coordinates = c0
                 else:
                     coordinates = 'MIX'
 
-                cplAz = azimuth(coordinates, X0, Y0, X1, Y1)
-                logging.info("Azimuth=%.1f"%cplAz)
+                cpl_az = azimuth(coordinates, x0, y0, x1, y1)
+                logger.info("Azimuth=%.1f"%cpl_az)
             else:
-                cplAz = 0.
+                cpl_az = 0.
 
             for components in params.components_to_compute:
-                logging.debug("Processing {!s}".format(components))
+                logger.debug("Processing {!s}".format(components))
                 t1 = stream.select(network=s1.net, station=s1.sta)
                 t2 = stream.select(network=s2.net, station=s2.sta)
                 if (components == "ZZ") \
@@ -290,7 +277,7 @@ def main():
                     t1 = t1.select(component=components[0])
                     t2 = t2.select(component=components[1])
                 else:
-                    logging.debug('Rotating streams, making sure they are'
+                    logger.debug('Rotating streams, making sure they are'
                                   ' aligned')
 
                     if components[0] == "Z":
@@ -302,7 +289,7 @@ def main():
                         if len(t1_novert):
                             # Make these streams contain the same gaps
                             t1_novert = make_same_length(t1_novert)
-                            t1 = t1_novert.rotate("NE->RT", cplAz).\
+                            t1 = t1_novert.rotate("NE->RT", cpl_az).\
                                 select(component=components[0])
                         else:
                             t1 = t1_novert
@@ -316,17 +303,17 @@ def main():
                         if len(t2_novert):
                             # Make these streams contain the same gaps
                             t2_novert = make_same_length(t2_novert)
-                            t2 = t2_novert.rotate("NE->RT", cplAz).\
+                            t2 = t2_novert.rotate("NE->RT", cpl_az).\
                                 select(component=components[1])
                         else:
                             t2 = t2_novert
 
                 if not len(t1):
-                    logging.info("No Data for %s.%s..%s" % (
+                    logger.info("No Data for %s.%s..%s" % (
                         s1.net, s1.sta, components[0]))
                     continue
                 if not len(t2):
-                    logging.info("No Data for %s.%s..%s" % (
+                    logger.info("No Data for %s.%s..%s" % (
                         s2.net, s2.sta, components[1]))
                     continue
 
@@ -342,7 +329,7 @@ def main():
                             gaps.append(gap)
 
                     if len(gaps) > 0:
-                        logging.debug("Sliding Windows %s contains gaps,"
+                        logger.debug("Sliding Windows %s contains gaps,"
                                       " skipping..." % tmp[0].stats.starttime)
                         continue
                     if tmp[0].stats.npts < 2*(params.maxlag *
@@ -359,6 +346,8 @@ def main():
                         elif params.windsorizing != 0:
                             rms = tr.data.std() * params.windsorizing
                             np.clip(tr.data, -rms, rms, tr.data)  # inplace
+
+                    # TODO should not hardcode 4 percent!
                     tmp.taper(0.04)
                     tmp1 = tmp.select(network=s1.net, station=s1.sta,
                                       component=components[0])
@@ -385,7 +374,7 @@ def main():
                     elif params.whitening == "N":
                         whitening = False
 
-                    for filterdb in get_filters(db, all=False):
+                    for filterdb in filters:
                         filterid = filterdb.ref
                         low = float(filterdb.low)
                         high = float(filterdb.high)
@@ -396,27 +385,27 @@ def main():
                         for i, station in enumerate(pair):
                             if tmp[i].data.std() > rms_threshold:
                                 if whitening:
-                                    #logging.debug("Whitening %s" % components)
+                                    #logger.debug("Whitening %s" % components)
                                     trames2hWb[i] = whiten(tmp[i].data, nfft,
                                                            dt, low, high,
                                                            plot=False)
                                 else:
-                                    #logging.debug("Autocorr %s"%components)
+                                    #logger.debug("Autocorr %s"%components)
                                     X = tmp[i].copy()
                                     X.filter("bandpass", freqmin=low,
                                              freqmax=high, zerophase=True)
-                                    trames2hWb[i] = scipy.fftpack.fft(X.data, nfft)
+                                    trames2hWb[i] = sf.fft(X.data, nfft)
                             else:
                                 skip = True
-                                logging.debug('Slice RMS is smaller (%e) than rms_threshold (%e)!'
+                                logger.debug('Slice RMS is smaller (%e) than rms_threshold (%e)!'
                                               % (tmp[i].data.std(), rms_threshold))
                         if not skip:
                             corr = myCorr(trames2hWb, np.ceil(params.maxlag / dt), plot=False, nfft=nfft)
                             if not np.all(np.isfinite(corr)):
-                                logging.debug("corr object contains NaNs, skipping")
+                                logger.debug("corr object contains NaNs, skipping")
                                 continue
                             if len(corr) < 2 * (params.maxlag * params.goal_sampling_rate) + 1:
-                                logging.debug(
+                                logger.debug(
                                     "corr object is too small, skipping")
                                 continue
                             tmptime = tmp[0].stats.starttime.datetime
@@ -443,7 +432,9 @@ def main():
                         station1, station2, filterid, components, date = ccfid.split('_')
 
                         corrs = np.asarray(list(allcorr[ccfid].values()))
-                        corr = stack(db, corrs)
+                        corr = stack(corrs, params.stack_method,
+                                     params.pws_timegate, params.pws_power,
+                                     params.goal_sampling_rate)
 
                         thisdate = goal_day
                         thistime = "0_0"
@@ -454,18 +445,20 @@ def main():
                                 params.goal_sampling_rate,
                                 components, corr,
                                 params.goal_sampling_rate, day=True,
-                                ncorr=corrs.shape[0])
+                                ncorr=corrs.shape[0], params=params)
                         del corrs, corr, thisdate, thistime
                 del current, allcorr, t1, t2
-            logging.debug("Updating Job")
-            update_job(db, goal_day, orig_pair, 'CC', 'D')
+            logger.debug("Updating Job")
+            # Would be better after the massive update at the end of the day job
+            # but here it allows to only insert the stack job if the CC was 
+            # successful.
+            if not params.hpc:
+                update_job(db, goal_day, orig_pair, 'STACK', 'T')
 
-            logging.info("Finished processing this pair. It took %.2f seconds" % (time.time() - tt))
+            logger.info("Finished processing this pair. It took %.2f seconds" % (time.time() - tt))
+        massive_update_job(db, jobs, "D")
         clean_scipy_cache()
-        logging.info("Job Finished. It took %.2f seconds" % (time.time() - jt))
+
+        logger.info("Job Finished. It took %.2f seconds (preprocess: %.2f s & process %.2f s)" % ((time.time() - jt), start_processing-jt, time.time()-start_processing))
         del stream
-    logging.info('*** Finished: Compute CC ***')
-
-if __name__ == "__main__":
-    main()    
-
+    logger.info('*** Finished: Compute CC ***')
