@@ -9,24 +9,26 @@ The corresponding configuration bits are ``ref_begin`` and ``ref_end``. In the
 future, we plan on allowing multiple references to be defined.
 
 Only data for new/modified dates need to be exported. If any CC-job has been
-marked "Done" within the last day, the stacks will be calculated and a new DTT
-job will be inserted in the database. For dates in the period of interest, the
-moving-window stack will only be exported if new/modified CCF is available.
+marked "Done" within the last day and triggered the creation of STACK jobs,
+the stacks will be calculated and a new MWCS job will be inserted in the
+database. For dates in the period of interest, the moving-window stack will
+only be exported if new/modified CCF is available.
 The export directory are "REF/" and "DAY%03i/" where %03i will be replaced by
 the number of days stacked together (DAYS_005 for a 5-days stack, e.g.).
 
 Please note that within MSNoise, stacks are always *inclusive* of the time/day
-mentionned. For example, a 5-days stack on January 10, will contain
+mentioned. For example, a 5-days stack on January 10, will contain
 cross-correlation functions computed for January 6, 7, 8, 9 AND 10!
 The graphical representation centered on a "January 10" tick might then display
 changes in the CCF that occurred *on* the 10th !
 
-Moving-window stack length(s) are configured using the ``mov_stack`` bit.
+Moving-window stacks are configured using the ``mov_stack`` parameter in
+``msnoise admin``.
 
 If ``stack_method`` is 'linear', then a simple mean CFF of all daily is saved
 as the mov or ref CCF. On the other hand, if ``stack_method`` is 'pws', then
 all the Phase Weighted Stack (PWS) is computed and saved as the mov or ref CCF.
-The PWS is done in two steps: first the mean coherence between the instataneous
+The PWS is done in two steps: first the mean coherence between the instantaneous
 phases of all windows is calculated, and eventually serves a weighting factor
 on the mean. The smoothness of this weighting array is defined using the
 ``pws_timegate`` parameter in the configuration. The weighting array is the
@@ -51,50 +53,47 @@ Configuration Parameters
 * |stack_method| | *new in 1.4*
 * |pws_timegate| | *new in 1.4*
 * |pws_power| | *new in 1.4*
+* |hpc| | *new in 1.6*
 
 
+Once done, each job is marked "D"one in the database and, unless ``hpc`` is 
+``Y``, MWCS jobs are inserted/updated in the database.
+
 Usage:
 ~~~~~~
-The best way to call this code is to start it from the console (-h shows the
-help)
+
+.. include:: ../clickhelp/msnoise-stack.rst
+
+
+For most users, the REF stack will need to be computed only once for specific
+dates and then, on routine basis, only compute the MOV stacks:
+
+.. warning With MSNoise 1.6, we have split the two actions, and the REF
+    stacks need to be computed first ! This process will put the corresponding
+    STACK jobs "I"n progress and you will need to reset them before running the
+    MOV stacks.
 
 .. code-block:: sh
 
-    $ msnoise stack --help
+    $ msnoise stack -r
+    $ msnoise reset STACK
+    $ msnoise stack -m
 
-    Usage: msnoise-script.py stack [OPTIONS]
-
-      Stacks the [REF] and/or [MOV] windows
-
-    Options:
-      -r, --ref               Compute the REF Stack
-      -m, --mov               Compute the MOV Stacks
-      -s, --step              Compute the STEP Stacks
-      -i, --interval INTEGER  Number of days before now to search for modified
-                              Jobs
-      --help                  Show this message and exit.
-
-On a routine basis, one should thus run the following to compute REF *and* MOV
-stacks:
+as for all other steps, this procedure can be run in parallel:
 
 .. code-block:: sh
 
-    $ msnoise stack -r -m
-
-While, when playing around with data, and surely on the first run, one should
-define the *-i INTERVAL*, as jobs might have been marked "Done" more than 24
-hours before running the stack. This, for example, will tell the code to search
-for jobs marked in the last 10 days:
-
-.. code-block:: sh
-
-    $ msnoise stack -r -m -i 10
-
-
+    $ msnoise -t 4 stack -r
+    $ msnoise reset STACK
+    $ msnoise -t 4 stack -m
 
 
 .. versionadded:: 1.4
     The Phase Weighted Stack.
+
+.. versionadded:: 1.6
+    The ``hpc`` parameter that can prevent the automatic creation of MWCS jobs.
+    The REF and MOV stacks have been separated and need to be run independently.
 """
 
 import argparse
@@ -104,7 +103,10 @@ import scipy.signal
 from .api import *
 
 
-def main(stype, interval=1.0):
+import logbook
+
+
+def main(stype, interval=1.0, loglevel="INFO"):
     """Computes the REF/MOV stacks.
     
     Parameters
@@ -115,14 +117,13 @@ def main(stype, interval=1.0):
         Number of days before now to search for modified CC jobs
 
     """
-    logging.basicConfig(level=logging.DEBUG,
-                    format='%(asctime)s [%(levelname)s] %(message)s',
-                    datefmt='%Y-%m-%d %H:%M:%S')
-
-    
-    logging.debug('Starting the %s stack' % stype)
+    logger = logbook.Logger(__name__)
+    # Reconfigure logger to show the pid number in log records
+    logger = get_logger('msnoise.stack_child', loglevel,
+                        with_pid=True)
+    logger.debug('Starting the %s stack' % stype)
     db = connect()
-    components_to_compute = get_components_to_compute(db)
+    
     export_format = get_config(db, 'export_format')
 
     if export_format == "BOTH":
@@ -142,7 +143,8 @@ def main(stype, interval=1.0):
     pws_timegate = float(get_config(db, 'pws_timegate'))
     pws_power = float(get_config(db, 'pws_power'))
     goal_sampling_rate = float(get_config(db, "cc_sampling_rate"))
-
+    # Get Configuration
+    params = get_params(db)
     plugins = get_config(db, "plugins")
     extra_jobtypes = []
     if plugins:
@@ -154,7 +156,6 @@ def main(stype, interval=1.0):
                 for jobtype in jobtypes:
                     if jobtype["after"] == "refstack":
                         extra_jobtypes.append(jobtype["name"])
-    print(extra_jobtypes)
     
     if stype == "mov" or stype == "step":
         start, end, datelist = build_movstack_datelist(db)
@@ -184,26 +185,46 @@ def main(stype, interval=1.0):
                 rng = pd.date_range(start, end, freq="%iD"%mov_stack)
             datelists[mov_stack] = rng.map(lambda t: t.date())
         #~ print datelists
+    biglist = []
+    filters = get_filters(db, all=False)
+    while is_dtt_next_job(db, flag='T', jobtype='STACK'):
+        jobs = get_dtt_next_job(db, flag='T', jobtype='STACK')
 
-    for f in get_filters(db, all=False):
-        filterid = int(f.ref)
-        for components in components_to_compute:
-            for station1, station2 in get_station_pairs(db, used=True):
-                sta1 = "%s_%s" % (station1.net, station1.sta)
-                sta2 = "%s_%s" % (station2.net, station2.sta)
+        if not len(jobs):
+            # edge case, should only occur when is_next returns true, but
+            # get_next receives no jobs (heavily parallelised calls).
+            time.sleep(np.random.random())
+            continue
+        pair = jobs[0].pair
+        refs, days = zip(*[[job.ref, job.day] for job in jobs])
+
+        logger.info(
+            "There are STACKS jobs for some days to recompute for %s" % pair)
+        sta1, sta2 = pair.split(':')
+        for f in filters:
+            filterid = int(f.ref)
+            for components in params.all_components:
                 pair = "%s:%s" % (sta1, sta2)
-                logging.debug('Processing %s-%s-%i' %
+                sta1 = sta1.replace('.', '_')
+                sta2 = sta2.replace('.', '_')
+                logger.debug('Processing %s-%s-%i' %
                                   (pair, components, filterid))
-                updated_days = updated_days_for_dates(db, start, end, pair.replace('_', '.'), jobtype='CC', interval=datetime.timedelta(days=interval),returndays=True)
+                # updated_days = updated_days_for_dates(db, start, end, pair.replace('_', '.'), jobtype='CC', interval=datetime.timedelta(days=interval),returndays=True)
+                updated_days = [UTCDateTime(d).datetime.date() for d in days]
                 if len(updated_days) != 0:
-                    logging.debug("New Data for %s-%s-%i" %
+                    logger.debug("New Data for %s-%s-%i" %
                                   (pair, components, filterid))
                     #~ print updated_days
+                    # TODO: load only the updated dates +- max(mov_stack)+1
+                    # Note: this would no longer be needed if the stack is h5
                     nstack, stack_total = get_results(
-                        db, sta1, sta2, filterid, components, datelist, format=format)
-                    logging.debug("Data loaded")
+                        db, sta1, sta2, filterid, components, datelist, format=format, params=params)
+                    if not nstack:
+                        logger.debug("No new data found, hmmm")
+                    logger.debug("Data loaded")
                     if nstack > 0:
                         if stype == "mov":
+                            logger.debug("Mov Stack!")
                             for i, date in enumerate(datelist):
                                 jobadded = False
                                 for mov_stack in mov_stacks:
@@ -223,31 +244,36 @@ def main(stype, interval=1.0):
                                         if not np.all(np.isnan(corr)):
                                             day_name = "%s_%s" % (
                                                 sta1, sta2)
-                                            logging.debug("%s %s [%s - %s] (%i day stack)" % (
-                                                day_name, date, datelist[low], datelist[i], mov_stack))
-                                            corr = stack(db, corr)
+                                            logger.debug("%s %s %s [%s - %s] (%i day stack)" % (
+                                                day_name, components, date, datelist[low], datelist[i], mov_stack))
+                                            corr = stack(corr, stack_method,
+                                                         pws_timegate,
+                                                         pws_power,
+                                                         goal_sampling_rate)
+                                            if not len(corr):
+                                                continue
 
                                             corr = scipy.signal.detrend(
-                                                corr)
+                                                corr).astype(np.float32)
                                             stack_path = os.path.join(
                                                 "STACKS", "%02i" % filterid, "%03i_DAYS" % mov_stack, components, day_name)
                                             filename = os.path.join(
                                                 stack_path, str(date))
                                             if mseed:
                                                 export_mseed(
-                                                    db, filename, pair, components, filterid, corr, maxlag=maxlag, cc_sampling_rate=cc_sampling_rate)
+                                                    db, filename, pair, components, filterid, corr, maxlag=maxlag, cc_sampling_rate=cc_sampling_rate, params=params)
                                             if sac:
                                                 export_sac(
-                                                    db, filename, pair, components, filterid, corr, maxlag=maxlag, cc_sampling_rate=cc_sampling_rate)
+                                                    db, filename, pair, components, filterid, corr, maxlag=maxlag, cc_sampling_rate=cc_sampling_rate, params=params)
                                             day_name = "%s:%s" % (
                                                 sta1, sta2)
-                                            if not jobadded:
+                                            if not jobadded and not params.hpc:
                                                 update_job(
-                                                    db, date, day_name.replace('_', '.'), 'DTT', 'T')
+                                                    db, date, day_name.replace('_', '.'), 'MWCS', 'T')
                                                 jobadded = True
                                         del corr
                         elif stype == "step":
-                            jobs = []
+                            jobs_done = []
                             for mov_stack in mov_stacks:
                                 for i, date in enumerate(datelists[mov_stack]):
                                     if date not in datelist:
@@ -268,9 +294,12 @@ def main(stype, interval=1.0):
                                         if not np.all(np.isnan(corr)):
                                             day_name = "%s_%s" % (
                                                 sta1, sta2)
-                                            logging.debug("%s %s [%s - %s] (%i day stack)" % (
-                                                day_name, date, datelist[low], datelist[high-1], mov_stack))
-                                            corr = stack(db, corr)
+                                            logger.debug("%s %s %s [%s - %s] (%i day stack)" % (
+                                                day_name, components, date, datelist[low], datelist[high-1], mov_stack))
+                                            corr = stack(corr, stack_method,
+                                                         pws_timegate,
+                                                         pws_power,
+                                                         goal_sampling_rate)
                                             corr = scipy.signal.detrend(corr)
                                             stack_path = os.path.join(
                                                 "STACKS", "%02i" % filterid, "%03i_DAYS" % mov_stack, components, day_name)
@@ -278,27 +307,19 @@ def main(stype, interval=1.0):
                                                 stack_path, str(date))
                                             if mseed:
                                                 export_mseed(
-                                                    db, filename, pair, components, filterid, corr, maxlag=maxlag, cc_sampling_rate=cc_sampling_rate)
+                                                    db, filename, pair, components, filterid, corr, maxlag=maxlag, cc_sampling_rate=cc_sampling_rate, params=params)
                                             if sac:
                                                 export_sac(
-                                                    db, filename, pair, components, filterid, corr, maxlag=maxlag, cc_sampling_rate=cc_sampling_rate)
+                                                    db, filename, pair, components, filterid, corr, maxlag=maxlag, cc_sampling_rate=cc_sampling_rate, params=params)
                                             day_name = "%s:%s" % (
                                                 sta1, sta2)
                                             job = "%s %s" % (date, day_name)
-                                            if job not in jobs:
+                                            if job not in jobs_done and not params.hpc:
                                                 update_job(
-                                                    db, date, day_name.replace('_', '.'), 'DTT', 'T')
-                                                jobs.append(job)
+                                                    db, date, day_name.replace('_', '.'), 'MWCS', 'T')
+                                                jobs_done.append(job)
                                         del corr
-                            #~ for date in datelist:
-                                #~ day_name = "%s:%s" % (
-                                                    #~ sta1, sta2)
-                                #~ job = "%s %s" % (date, day_name)
-                                #~ if job not in jobs:
-                                    #~ update_job(
-                                        #~ db, date, day_name.replace('_', '.'), 'DTT', 'T')
-                                    #~ jobs.append(job)
-                        
+
                         elif stype == "ref":
                             stack_path = os.path.join(
                                 "STACKS", "%02i" % filterid, "REF", components)
@@ -308,54 +329,33 @@ def main(stype, interval=1.0):
 
                             if mseed:
                                 export_mseed(
-                                    db, filename, pair, components, filterid, stack_total)
+                                    db, filename, pair, components, filterid, stack_total, params=params)
                             if sac:
                                 export_sac(
-                                    db, filename, pair, components, filterid, stack_total)
+                                    db, filename, pair, components, filterid, stack_total,params=params)
                             ref_name = "%s:%s" % (sta1, sta2)
                             update_job(
-                                db, "REF", ref_name.replace('_', '.'), 'DTT', 'T')
+                                db, "REF", ref_name.replace('_', '.'), 'MWCS', 'T')
                             for jobtype in extra_jobtypes:
                                 update_job(db, "REF", ref_name.replace('_', '.'), jobtype, 'T')
                             del stack_total
 
+        # THIS SHOULD BE IN THE API
+        # This doesn't set MWCS jobs for REF stacks
+        if stype != "ref":
+            massive_update_job(db, jobs, "D")
+            if stype != "step" and not params.hpc:
+                for job in jobs:
+                    update_job(db, job.day, job.pair, 'MWCS', 'T')
+        if stype == "ref":
+            biglist += jobs
 
-def refstack(interval):
-    main('ref', interval)
+    if stype == "ref":
+        logger.info("You just finished REF stacking, remember to reset the "
+                    "STACK jobs if you need to compute a MOV stacks. "
+                    "Run 'msnoise reset STACK' when all process have finished.")
+        logger.info("The current STACK jobs have been intentionnaly left "
+                    "'I'n progress so they can be reset.")
+    #     massive_update_job(db, biglist, "T")
 
-
-def movstack(interval):
-    main('mov', interval)
-
-
-if __name__ == "__main__":
-    logging.basicConfig(level=logging.DEBUG,
-                    format='%(asctime)s [%(levelname)s] %(message)s',
-                    datefmt='%Y-%m-%d %H:%M:%S')
-
-    parser = argparse.ArgumentParser(description='Compute [REF,MOV] stacks if\
-                                jobs have been modified in the last i days.',
-                                     epilog=__doc__)
-    parser.add_argument('-r', '--ref', action="store_true",
-                        help='Triggers the computation of REF stacks',
-                        default=False)
-    parser.add_argument('-m', '--mov', action="store_true",
-                        help='Triggers the computation of MOV stacks',
-                        default=False)
-    parser.add_argument('-i', '--interval',
-                        help='Number of days before now to search for\
-                        modified CC jobs [default:1]', default=1, type=int)
-    args = parser.parse_args()
-
-    logging.info('Starting this program with: ref=%s, mov=%s, interval=%i'
-                 % (args.ref, args.mov, args.interval))
-
-    db = connect()
-    if args.ref:
-        logging.info("*** Starting: REF Stack ***")
-        refstack(args.interval)
-        logging.info("*** Finished: REF Stack ***")
-    if args.mov:
-        logging.info("*** Starting: MOV Stack ***")
-        movstack(args.interval)
-        logging.info("*** Finished: MOV Stack ***")
+    logger.debug("Finished Stacking")
