@@ -2,12 +2,6 @@
 This code is responsible for the calculation of dt/t using the result of the
 MWCS calculations.
 
-.. warning:: Previously, all pairs were analysed using the same parameters,
-    which were hard-coded in the s06compute_dtt.py file.
-    This has changed now, and MSNoise uses parameters set in the database via
-    the configurator. Pre-1.3 users should upgrade their database using the
-    "$ msnoise upgrade_db" command.
-
 
 Configuration Parameters
 ~~~~~~~~~~~~~~~~~~~~~~~~
@@ -110,27 +104,32 @@ is saved to the output file, as visible above. In the future, MSNoise will be
 able to treat as many groups as the user want, allowing, e.g. a "crater" and
 a "slopes" groups.
 
+Forcing vs No Forcing through Origin
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+The reason for allowing the WLS to cross the axis elsewhere than on (0,0) is,
+for example, to study the potential clock drifts or noise source position
+variations. By default, the ``msnoise plot dvv`` plot shows the results
+of the ```Not Forced`` WLS.
+
 Mean of All Pairs vs Mean Pair
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+.. warning:: the ``ALL`` pair is still calculated and output in the DTT files,
+    but its output is no longer displayed on the graphs. *new in 1.6*.
+
 The dt/t calculated using the mean pair (ALL, in red on subplots 4 and 5)
 and by calculating the weighted mean of the dt/t of all pairs (in green)
 don't show a significant difference. The standard deviation around the latter
 is more spread than on the former, but this has to be investigated.
 
-Forcing vs No Forcing through Origin
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-The reason for allowing the WLS to cross the axis elsewhere than on (0,0) is,
-for example, to study the potential clock drifts or noise source position
-variations.
 
 """
 
+from obspy.signal.regression import linear_regression
 from .api import *
 
-try:
-    from obspy.signal.regression import linear_regression
-except ImportError:
-    from .api import linear_regression
+
+import logbook
 
 
 def wavg_wstd(data, errors):
@@ -143,84 +142,94 @@ def wavg_wstd(data, errors):
     return wavg, wstd
 
 
-def main(interval=1):
-    logging.basicConfig(level=logging.DEBUG,
-                        format='%(asctime)s [%(levelname)s] %(message)s',
-                        datefmt='%Y-%m-%d %H:%M:%S')
-
-    logging.info('*** Starting: Compute DT/T ***')
+def main(interval=1, loglevel="INFO"):
+    logger = logbook.Logger(__name__)
+    # Reconfigure logger to show the pid number in log records
+    logger = get_logger('msnoise.compute_dtt_child', loglevel,
+                        with_pid=True)
+    logger.info('*** Starting: Compute DT/T ***')
     db = connect()
-    
-    dtt_lag = get_config(db, "dtt_lag")
-    dtt_v = float(get_config(db, "dtt_v"))
-    dtt_minlag = float(get_config(db, "dtt_minlag"))
-    dtt_width = float(get_config(db, "dtt_width"))
-    dtt_sides = get_config(db, "dtt_sides")
-    minCoh = float(get_config(db, "dtt_mincoh"))
-    maxErr = float(get_config(db, "dtt_maxerr"))
-    maxDt = float(get_config(db, "dtt_maxdt"))
-    
+    params = get_params(db)
+
     start, end, datelist = build_movstack_datelist(db)
-    
+
     mov_stack = get_config(db, "mov_stack")
     if mov_stack.count(',') == 0:
         mov_stacks = [int(mov_stack), ]
     else:
         mov_stacks = [int(mi) for mi in mov_stack.split(',')]
-    
+
     components_to_compute = get_components_to_compute(db)
     updated_dtt = updated_days_for_dates(
         db, start, end, '%', jobtype='DTT', returndays=True,
         interval=datetime.timedelta(days=interval))
-    
-    for f in get_filters(db, all=False):
-        filterid = int(f.ref)
-        for components in components_to_compute:
-            for mov_stack in mov_stacks:
-                logging.info('Loading mov=%i days for filter=%02i' %
-                             (mov_stack, filterid))
-                for current in updated_dtt:
-                    if current > datetime.date.today():
-                        break
-                    logging.debug("Processing %s - %02i - %02i mov" %
-                                  (current, filterid, mov_stack))
+    interstations = {}
+    for sta1, sta2 in get_station_pairs(db):
+        s1 = "%s_%s" % (sta1.net, sta1.sta)
+        s2 = "%s_%s" % (sta2.net, sta2.sta)
+        if s1 == s2:
+            interstations["%s_%s" % (s1, s2)] = 0.0
+        else:
+            interstations["%s_%s"%(s1,s2)] = get_interstation_distance(sta1,
+                                                                       sta2,
+                                                                       sta1.coordinates)
+        
+    filters = get_filters(db, all=False)
+    while is_next_job(db, jobtype='DTT'):
+        jobs = get_next_job(db, jobtype='DTT')
+
+        stations = []
+        pairs = []
+        refs = []
+        for f in filters:
+            filterid = int(f.ref)
+            for components in params.all_components:
+                for mov_stack in mov_stacks:
+                    logger.info('Loading mov=%i days for filter=%02i' %
+                                 (mov_stack, filterid))
                     first = True
-                    for station1, station2 in get_station_pairs(db, used=True):
-                        sta1 = "%s_%s" % (station1.net, station1.sta)
-                        sta2 = "%s_%s" % (station2.net, station2.sta)
+                    for job in jobs:
+                        refs.append(job.ref)
+                        pairs.append(job.pair)
+                        netsta1, netsta2 = job.pair.split(':')
+                        stations.append(netsta1)
+                        stations.append(netsta2)
+                        current = job.day
+                        sta1 = netsta1.replace(".", "_")
+                        sta2 = netsta2.replace(".", "_")
                         pair = "%s_%s" % (sta1, sta2)
                         day = os.path.join('MWCS', "%02i" % filterid,
                                            "%03i_DAYS" % mov_stack, components,
                                            pair, '%s.txt' % current)
-                        dist = get_interstation_distance(station1, station2,
-                                                         station1.coordinates)
-                        if dist == 0. and dtt_lag == "dynamic":
-                            logging.debug('%s: Distance is Zero?!' % pair)
+                        # dist = get_interstation_distance(station1, station2,
+                        #                                  station1.coordinates)
+                        dist = interstations[pair]
+                        if dist == 0. and params.dtt_lag == "dynamic":
+                            logger.debug('%s: Distance is Zero?!' % pair)
                         if os.path.isfile(day):
                             df = pd.read_csv(
                                 day, delimiter=' ', header=None, index_col=0,
                                 names=['t', 'dt', 'err', 'coh'])
                             tArray = df.index.values
-                            if dtt_lag == "static":
-                                lmlag = -dtt_minlag
-                                rmlag = dtt_minlag
+                            if params.dtt_lag == "static":
+                                lmlag = -params.dtt_minlag
+                                rmlag = params.dtt_minlag
                             else:
-                                lmlag = -dist / dtt_v
-                                rmlag = dist / dtt_v
-                            lMlag = lmlag - dtt_width
-                            rMlag = rmlag + dtt_width
+                                lmlag = -dist / params.dtt_v
+                                rmlag = dist / params.dtt_v
+                            lMlag = lmlag - params.dtt_width
+                            rMlag = rmlag + params.dtt_width
 
-                            if dtt_sides == "both":
+                            if params.dtt_sides == "both":
                                 tindex = np.where(((tArray >= lMlag) & (tArray <= lmlag)) | ((tArray >= rmlag) & (tArray <= rMlag)))[0]
-                            elif dtt_sides == "left":
+                            elif params.dtt_sides == "left":
                                 tindex = np.where((tArray >= lMlag) & (tArray <= lmlag))[0]
                             else:
                                 tindex = np.where((tArray >= rmlag) & (tArray <= rMlag))[0]
                             
                             tmp = np.setdiff1d(np.arange(len(tArray)),tindex)
-                            df['err'][tmp] = 1.0
-                            df['coh'][tmp] = 0.0
-                            
+                            df.loc[:, 'err'].iloc[tmp] = 1.0
+                            df.loc[:, 'coh'].iloc[tmp] = 0.0
                             if first:
                                 tArray = df.index.values
                                 dtArray = df['dt']
@@ -258,11 +267,11 @@ def main(interval=1):
                                 #~ if i in tindex:
                                 if 1:
                                     cohindex = np.where(
-                                        cohArray[:, i] >= minCoh)[0]
+                                        cohArray[:, i] >= params.dtt_mincoh)[0]
                                     errindex = np.where(
-                                        errArray[:, i] <= maxErr)[0]
+                                        errArray[:, i] <= params.dtt_maxerr)[0]
                                     dtindex = np.where(
-                                        np.abs(dtArray[:, i]) <= maxDt)[0]
+                                        np.abs(dtArray[:, i]) <= params.dtt_maxdt)[0]
     
                                     index = np.intersect1d(cohindex, errindex)
                                     index = np.intersect1d(index, dtindex)
@@ -300,11 +309,11 @@ def main(interval=1):
                                     #~ if i in tindex:
                                     if 1:
                                         cohindex = np.where(
-                                            cohArray[:, i] >= minCoh)[0]
+                                            cohArray[:, i] >= params.dtt_mincoh)[0]
                                         errindex = np.where(
-                                            errArray[:, i] <= maxErr)[0]
+                                            errArray[:, i] <= params.dtt_maxerr)[0]
                                         dtindex = np.where(
-                                            np.abs(dtArray[:, i]) <= maxDt)[0]
+                                            np.abs(dtArray[:, i]) <= params.dtt_maxdt)[0]
         
                                         index = np.intersect1d(cohindex,
                                                                errindex)
@@ -329,16 +338,16 @@ def main(interval=1):
     
                         # then process all pairs + the ALL
                         if len(dtArray.shape) == 1:  # if there is only one pair:
-                            dtArray = dtArray.reshape((1, dtArray.shape[0]))
-                            cohArray = cohArray.reshape((1, cohArray.shape[0]))
-                            errArray = errArray.reshape((1, errArray.shape[0]))
+                            dtArray = dtArray.values.reshape((1, dtArray.shape[0]))
+                            cohArray = cohArray.values.reshape((1, cohArray.shape[0]))
+                            errArray = errArray.values.reshape((1, errArray.shape[0]))
     
                         used = np.zeros(dtArray.shape)
     
                         for i, pair in enumerate(pairArray):
-                            cohindex = np.where(cohArray[i] >= minCoh)[0]
-                            errindex = np.where(errArray[i] <= maxErr)[0]
-                            dtindex = np.where(np.abs(dtArray[i]) <= maxDt)[0]
+                            cohindex = np.where(cohArray[i] >= params.dtt_mincoh)[0]
+                            errindex = np.where(errArray[i] <= params.dtt_maxerr)[0]
+                            dtindex = np.where(np.abs(dtArray[i]) <= params.dtt_maxdt)[0]
     
                             #~ index = np.intersect1d(tindex, cohindex)
                             index = np.intersect1d(cohindex, errindex)
@@ -373,27 +382,37 @@ def main(interval=1):
                             del VecXfilt, VecYfilt, w
                             del index, cohindex, errindex, dtindex
     
-                        logging.debug(
+                        logger.debug(
                             "%s: exporting: %i pairs" % (current,
                                                          len(pairArray)))
                         df = pd.DataFrame(
                             {'Pairs': Pairs, 'M': M, 'EM': EM, 'A': A, 'EA': EA,
                              'M0': M0, 'EM0': EM0},
-                            index=pd.DatetimeIndex(Dates))
+                            index=Dates)
                         # Needs to be changed !
                         output = os.path.join(
                             'DTT', "%02i" % filterid, "%03i_DAYS" % mov_stack,
                             components)
                         if not os.path.isdir(output):
                             os.makedirs(output)
-                        df.to_csv(
-                            os.path.join(output, '%s.txt' % current),
-                            index_label='Date')
+                        fn = os.path.join(output, '%s.txt' % current)
+                        if os.path.isfile(fn):
+                            existing = pd.read_csv(fn, index_col="Pairs", parse_dates=True)
+                            for id, row in df.iterrows():
+                                if row.Pairs in existing.index.values:
+                                    existing.drop(row.Pairs, inplace=True)
+                                    logger.debug("Pair: %s is already in the output file, overwriting" % row.Pairs)
+                            existing["Pairs"] = existing.index.values
+                            existing.set_index("Date", inplace=True)
+                            df = pd.concat([df, existing])
+                        df.to_csv(fn, index_label='Date')
                         del df, M, EM, A, EA, M0, EM0, Pairs, Dates, used
                         del tArray, dtArray, errArray, cohArray, pairArray
                         del output
+        # THIS SHOULD BE IN THE API
+        massive_update_job(db, jobs, "D")
 
-    logging.info('*** Finished: Compute DT/T ***')
+    logger.info('*** Finished: Compute DT/T ***')
 
 
 if __name__ == "__main__":
